@@ -364,11 +364,11 @@ static void config_compat_module_load (server *srv) {
     int append_mod_staticfile  = 1;
     int append_mod_authn_file  = 1;
     int append_mod_authn_ldap  = 1;
-    int append_mod_authn_mysql = 1;
     int append_mod_openssl     = 1;
     int contains_mod_auth      = 0;
     int prepend_mod_auth       = 0;
     int prepend_mod_vhostdb    = 0;
+    const char *dyn_name = NULL;
 
     for (uint32_t i = 0; i < srv->srvconf.modules->used; ++i) {
         buffer *m = &((data_string *)srv->srvconf.modules->data[i])->value;
@@ -390,8 +390,15 @@ static void config_compat_module_load (server *srv) {
         else if (buffer_eq_slen(m, CONST_STR_LEN("mod_wolfssl")))
             append_mod_openssl = 0;
         else if (0 == strncmp(m->ptr, "mod_auth", sizeof("mod_auth")-1)) {
-            if (buffer_eq_slen(m, CONST_STR_LEN("mod_auth")))
-                contains_mod_auth = 1;
+            if (buffer_eq_slen(m, CONST_STR_LEN("mod_auth"))) {
+                if (!contains_mod_auth) {
+                    contains_mod_auth = 1;
+                    if (dyn_name)
+                        log_error(srv->errh, __FILE__, __LINE__,
+                          "Warning: mod_auth should be listed in server.modules"
+                          " before dynamic backends such as %s", dyn_name);
+                }
+            }
             else if (!contains_mod_auth)
                 prepend_mod_auth = 1;
 
@@ -399,8 +406,6 @@ static void config_compat_module_load (server *srv) {
                 append_mod_authn_file = 0;
             else if (buffer_eq_slen(m, CONST_STR_LEN("mod_authn_ldap")))
                 append_mod_authn_ldap = 0;
-            else if (buffer_eq_slen(m, CONST_STR_LEN("mod_authn_mysql")))
-                append_mod_authn_mysql = 0;
         }
         else if (0 == strncmp(m->ptr, "mod_vhostdb", sizeof("mod_vhostdb")-1)) {
             if (buffer_eq_slen(m, CONST_STR_LEN("mod_vhostdb")))
@@ -422,11 +427,8 @@ static void config_compat_module_load (server *srv) {
                                          sizeof("mod_sockproxy")-1)
                  || 0 == strncmp(m->ptr, "mod_wstunnel",
                                          sizeof("mod_wstunnel")-1)) {
-            if (!contains_mod_auth) {
-                log_error(srv->errh, __FILE__, __LINE__,
-                  "Warning: mod_auth should be listed in server.modules before "
-                  "dynamic backends such as %s", m->ptr);
-            }
+            if (NULL == dyn_name)
+                dyn_name = m->ptr;
         }
     }
 
@@ -474,13 +476,6 @@ static void config_compat_module_load (server *srv) {
                 config_warn_authn_module(srv, CONST_STR_LEN("ldap"), "ldap");
           #endif
         }
-        if (append_mod_authn_mysql) {
-          #if defined(HAVE_MYSQL)
-            if (config_has_opt_and_value(srv, CONST_STR_LEN("auth.backend"),
-                                              CONST_STR_LEN("mysql")))
-                config_warn_authn_module(srv, CONST_STR_LEN("mysql"), "mysql");
-          #endif
-        }
     }
 
     if (prepend_mod_auth) {
@@ -1453,7 +1448,6 @@ void config_free(server *srv) {
     array_free(srv->config_context);
     array_free(srv->srvconf.config_touched);
     array_free(srv->srvconf.modules);
-    buffer_free(srv->srvconf.modules_dir);
     array_free(srv->srvconf.upload_tempdirs);
   #ifdef HAVE_PCRE2_H
     if (NULL == srv->match_data) pcre2_match_data_free(srv->match_data);
@@ -1484,7 +1478,7 @@ void config_init(server *srv) {
       | HTTP_PARSEOPT_URL_NORMALIZE_PATH_DOTSEG_REMOVE;
 
     srv->srvconf.modules = array_init(16);
-    srv->srvconf.modules_dir = buffer_init_string(LIBRARY_DIR);
+    srv->srvconf.modules_dir = LIBRARY_DIR;
     srv->srvconf.upload_tempdirs = array_init(2);
 }
 
@@ -2144,7 +2138,8 @@ static int config_parse(server *srv, config_t *context, const char *source, cons
 
 	if (ret != -1 && context->ok) {
 		/* add an EOL at EOF, better than say sorry */
-		configparser(pParser, TK_EOL, buffer_init_string("(EOL)"), context);
+		buffer_copy_string((token = buffer_init()), "(EOL)");
+		configparser(pParser, TK_EOL, token, context);
 		if (context->ok) {
 			configparser(pParser, 0, NULL, context);
 		}
