@@ -27,10 +27,6 @@
 #include <errno.h>
 #include <string.h>
 
-#ifdef HAVE_SYS_FILIO_H
-# include <sys/filio.h>
-#endif
-
 #include "sys-socket.h"
 
 #define HTTP_LINGER_TIMEOUT 5
@@ -290,7 +286,7 @@ connection_write_chunkqueue (connection * const con, chunkqueue * const restrict
         /* Linux: put a cork into socket as we want to combine write() calls
          * but only if we really have multiple chunks including non-MEM_CHUNK
          * (or if multiple chunks and TLS), and only if TCP socket */
-        if (NULL != c || con->is_ssl_sock) {
+        if (NULL != c || (max_bytes > 16384 && con->is_ssl_sock)) {
             const int sa_family = sock_addr_get_family(&con->srv_socket->addr);
             if (sa_family == AF_INET || sa_family == AF_INET6) {
                 corked = 1;
@@ -440,8 +436,7 @@ static int connection_handle_write(request_st * const r, connection * const con)
 	case -1: /* error on our side */
 		log_error(r->conf.errh, __FILE__, __LINE__,
 		  "connection closed: write failed on fd %d", con->fd);
-		connection_set_state_error(r, CON_STATE_ERROR);
-		return CON_STATE_ERROR;
+		__attribute_fallthrough__
 	case -2: /* remote close */
 		connection_set_state_error(r, CON_STATE_ERROR);
 		return CON_STATE_ERROR;
@@ -610,6 +605,7 @@ static chunk * connection_read_header_more(connection *con, chunkqueue *cq, chun
 }
 
 
+__attribute_cold__
 static void
 connection_transition_h2 (request_st * const h2r, connection * const con)
 {
@@ -623,7 +619,7 @@ connection_transition_h2 (request_st * const h2r, connection * const con)
     /* (h2r->state == CON_STATE_READ) for transition by ALPN
      *   or starting cleartext HTTP/2 with Prior Knowledge
      *   (e.g. via HTTP Alternative Services)
-     * (h2r->state == CON_STATE_RESPONSE_END) for Upgrade: h2c */
+     * (h2r->state == CON_STATE_REQUEST_END) for Upgrade: h2c */
 
     if (h2r->state != CON_STATE_ERROR)
         connection_set_state(h2r, CON_STATE_WRITE);
@@ -694,7 +690,6 @@ static int connection_handle_read_state(connection * const con)  {
                       "oversized request-header -> sending Status 431");
             r->http_status = 431; /* Request Header Fields Too Large */
             r->keep_alive = 0;
-            connection_set_state(r, CON_STATE_REQUEST_END);
             return 1;
         }
 
@@ -719,13 +714,13 @@ static int connection_handle_read_state(connection * const con)  {
             /* expecting ASCII method beginning with alpha char
              * or HTTP/2 pseudo-header beginning with ':' */
             /*(TLS handshake begins with SYN 0x16 (decimal 22))*/
-            log_error(r->conf.errh, __FILE__, __LINE__, "%s",
+            log_error(r->conf.errh, __FILE__, __LINE__, "%s (%s)",
                       c->mem->ptr[c->offset] == 0x16
                       ? "unexpected TLS ClientHello on clear port"
-                      : "invalid request-line -> sending Status 400");
+                      : "invalid request-line -> sending Status 400",
+                      con->dst_addr_buf.ptr);
             r->http_status = 400; /* Bad Request */
             r->keep_alive = 0;
-            connection_set_state(r, CON_STATE_REQUEST_END);
             return 1;
         }
     } while ((c = connection_read_header_more(con, cq, c, clen)));
@@ -776,7 +771,6 @@ static int connection_handle_read_state(connection * const con)  {
                             hdrs, header_len, "fd:%d rqst: ", con->fd);
     http_request_headers_process(r, hdrs, hoff, con->proto_default_port);
     chunkqueue_mark_written(cq, r->rqst_header_len);
-    connection_set_state(r, CON_STATE_REQUEST_END);
 
     if (light_btst(r->rqst_htags, HTTP_HEADER_UPGRADE)
         && 0 == r->http_status
@@ -1018,13 +1012,13 @@ connection_state_machine_loop (request_st * const r, connection * const con)
 				}
 				break;
 			}
-			/*if (r->state != CON_STATE_REQUEST_END) break;*/
+			/*connection_set_state(r, CON_STATE_REQUEST_END);*/
 			__attribute_fallthrough__
 		case CON_STATE_REQUEST_END: /* transient */
-			ostate = (0 == r->reqbody_length)
+			connection_set_state(r,
+			  (0 == r->reqbody_length)
 			  ? CON_STATE_HANDLE_REQUEST
-			  : CON_STATE_READ_POST;
-			connection_set_state(r, ostate);
+			  : CON_STATE_READ_POST);
 			__attribute_fallthrough__
 		case CON_STATE_READ_POST:
 		case CON_STATE_HANDLE_REQUEST:
@@ -1034,10 +1028,7 @@ connection_state_machine_loop (request_st * const r, connection * const con)
 				break;
 			  case HANDLER_WAIT_FOR_EVENT:
 				return;
-			  case HANDLER_COMEBACK:
-				/* redo loop; will not match r->state */
-				ostate = CON_STATE_CONNECT;
-				continue;
+			  /*case HANDLER_COMEBACK:*//*(not expected)*/
 			  /*case HANDLER_ERROR:*/
 			  default:
 				connection_set_state_error(r, CON_STATE_ERROR);
@@ -1052,15 +1043,16 @@ connection_state_machine_loop (request_st * const r, connection * const con)
 			connection_set_state(r, CON_STATE_WRITE);
 			__attribute_fallthrough__
 		case CON_STATE_WRITE:
-			if (connection_handle_write_state(r, con)
-			    != CON_STATE_RESPONSE_END)
-				break;
+			if (connection_handle_write_state(r, con) == CON_STATE_WRITE)
+				return;
 			__attribute_fallthrough__
 		case CON_STATE_RESPONSE_END: /* transient */
 		case CON_STATE_ERROR:        /* transient */
 			if (r->http_version > HTTP_VERSION_1_1 && r != &con->request)
 				return;
 			connection_handle_response_end_state(r, con);
+			/*(make sure ostate will not match r->state)*/
+			ostate = CON_STATE_RESPONSE_END;/* != r->state */
 			break;
 		case CON_STATE_CLOSE:
 			/*(should not be reached by HTTP/2 streams)*/
@@ -1266,32 +1258,6 @@ connection_state_machine_h2 (request_st * const h2r, connection * const con)
 
             connection_state_machine_loop(r, con);
 
-            if (r->resp_header_len && !chunkqueue_is_empty(&r->write_queue)
-                && max_bytes
-                && (r->resp_body_finished
-                    || (r->conf.stream_response_body
-                        & (FDEVENT_STREAM_RESPONSE
-                          |FDEVENT_STREAM_RESPONSE_BUFMIN)))) {
-
-                uint32_t dlen = max_bytes > 32768 ? 32768 : (uint32_t)max_bytes;
-                dlen = h2_send_cqdata(r, con, &r->write_queue, dlen);
-                if (dlen) { /*(do not resched (spin) if swin empty window)*/
-                    max_bytes -= (off_t)dlen;
-                    if (!chunkqueue_is_empty(&r->write_queue))
-                        resched |= 1;
-                }
-            }
-
-            {
-                if (chunkqueue_is_empty(&r->write_queue)) {
-                    if (r->resp_body_finished && r->state == CON_STATE_WRITE) {
-                        connection_set_state(r, CON_STATE_RESPONSE_END);
-                        if (__builtin_expect( (r->conf.log_state_handling), 0))
-                            connection_state_machine_loop(r, con);
-                    }
-                }
-            }
-
           #if 0
             if (log_state_handling)
                 log_error(r->conf.errh, __FILE__, __LINE__,
@@ -1299,7 +1265,35 @@ connection_state_machine_h2 (request_st * const h2r, connection * const con)
                   connection_get_state(r->state));
           #endif
 
-            if (r->state==CON_STATE_RESPONSE_END || r->state==CON_STATE_ERROR) {
+            if (r->state < CON_STATE_WRITE)
+                continue;
+            /* else CON_STATE_WRITE, CON_STATE_RESPONSE_END, CON_STATE_ERROR */
+            else if (r->state == CON_STATE_WRITE) {
+                if (__builtin_expect((!chunkqueue_is_empty(&r->write_queue)), 1)
+                    && max_bytes
+                    && (r->resp_body_finished
+                        || (r->conf.stream_response_body
+                            & (FDEVENT_STREAM_RESPONSE
+                              |FDEVENT_STREAM_RESPONSE_BUFMIN)))) {
+                    uint32_t dlen =
+                      max_bytes > 32768 ? 32768 : (uint32_t)max_bytes;
+                    dlen = h2_send_cqdata(r, con, &r->write_queue, dlen);
+                    if (dlen) { /*(do not resched (spin) if swin empty window)*/
+                        max_bytes -= (off_t)dlen;
+                        if (!chunkqueue_is_empty(&r->write_queue))
+                            resched |= 1;
+                    }
+                }
+                if (!chunkqueue_is_empty(&r->write_queue)
+                    || !r->resp_body_finished)
+                    continue;
+
+                connection_set_state(r, CON_STATE_RESPONSE_END);
+                if (__builtin_expect( (r->conf.log_state_handling), 0))
+                    connection_state_machine_loop(r, con);
+            }
+
+            {/*(r->state==CON_STATE_RESPONSE_END || r->state==CON_STATE_ERROR)*/
                 /*(trigger reschedule of con if frames pending)*/
                 if (h2c->rused == sizeof(h2c->r)/sizeof(*h2c->r)
                     && !chunkqueue_is_empty(con->read_queue))
