@@ -180,14 +180,19 @@ const buffer * http_response_set_last_modified(request_st * const r, const unix_
 }
 
 
-int http_response_handle_cachable(request_st * const r, const buffer * const lmod, const unix_time64_t lmtime) {
-	if (!(r->rqst_htags
-	      & (light_bshift(HTTP_HEADER_IF_NONE_MATCH)
-	        |light_bshift(HTTP_HEADER_IF_MODIFIED_SINCE)))) {
-		return HANDLER_GO_ON;
-	}
+__attribute_pure__
+static int http_response_maybe_cachable (const request_st * const r) {
+    return (r->rqst_htags
+            & (light_bshift(HTTP_HEADER_IF_NONE_MATCH)
+              |light_bshift(HTTP_HEADER_IF_MODIFIED_SINCE)));
+}
 
-	const buffer *vb;
+
+int http_response_handle_cachable(request_st * const r, const buffer *lmod, const unix_time64_t lmtime) {
+	if (!http_response_maybe_cachable(r))
+		return HANDLER_GO_ON;
+
+	const buffer *vb, *etag;
 
 	/*
 	 * 14.26 If-None-Match
@@ -200,10 +205,12 @@ int http_response_handle_cachable(request_st * const r, const buffer * const lmo
 	 */
 
 	if ((vb = http_header_request_get(r, HTTP_HEADER_IF_NONE_MATCH,
-	                                  CONST_STR_LEN("If-None-Match")))) {
+	                                  CONST_STR_LEN("If-None-Match")))
+	    && (etag = http_header_response_get(r, HTTP_HEADER_ETAG,
+	                                        CONST_STR_LEN("ETag")))) {
 		/*(weak etag comparison must not be used for ranged requests)*/
 		int range_request = (0 != light_btst(r->rqst_htags, HTTP_HEADER_RANGE));
-		if (http_etag_matches(&r->physical.etag, vb->ptr, !range_request)) {
+		if (http_etag_matches(etag, vb->ptr, !range_request)) {
 			if (http_method_get_or_head(r->http_method)) {
 				r->http_status = 304;
 				return HANDLER_FINISHED;
@@ -215,7 +222,10 @@ int http_response_handle_cachable(request_st * const r, const buffer * const lmo
 		}
 	} else if (http_method_get_or_head(r->http_method)
 		   && (vb = http_header_request_get(r, HTTP_HEADER_IF_MODIFIED_SINCE,
-		                                    CONST_STR_LEN("If-Modified-Since")))) {
+		                                    CONST_STR_LEN("If-Modified-Since")))
+		   && (lmod
+		       || (lmod = http_header_response_get(r, HTTP_HEADER_LAST_MODIFIED,
+				                           CONST_STR_LEN("Last-Modified"))))) {
 		/* last-modified handling */
 		if (buffer_is_equal(lmod, vb)
 		    || !http_date_if_modified_since(BUF_PTR_LEN(vb), lmtime)) {
@@ -283,7 +293,6 @@ void http_response_reset (request_st * const r) {
     if (r->physical.path.ptr) { /*(skip for mod_fastcgi authorizer)*/
         buffer_clear(&r->physical.doc_root);
         buffer_clear(&r->physical.basedir);
-        buffer_clear(&r->physical.etag);
         buffer_reset(&r->physical.path);
         buffer_reset(&r->physical.rel_path);
     }
@@ -308,9 +317,9 @@ handler_t http_response_reqbody_read_error (request_st * const r, int http_statu
 }
 
 
-void http_response_send_file (request_st * const r, buffer * const path, stat_cache_entry *sce) {
-	if (NULL == sce
-	    || (sce->fd < 0 && __builtin_expect( (0 != sce->st.st_size), 0))) {
+void http_response_send_file (request_st * const r, const buffer * const path, stat_cache_entry *sce) {
+	if (__builtin_expect( (NULL == sce), 0)
+	    || (__builtin_expect( (sce->fd < 0), 0) && 0 != sce->st.st_size)) {
 		sce = stat_cache_get_entry_open(path, r->conf.follow_symlink);
 		if (NULL == sce) {
 			r->http_status = (errno == ENOENT) ? 404 : 403;
@@ -351,55 +360,45 @@ void http_response_send_file (request_st * const r, buffer * const path, stat_ca
 		return;
 	}
 
-	int allow_caching = (0 == r->http_status || 200 == r->http_status);
-
 	/* set response content-type, if not set already */
-
-	if (!light_btst(r->resp_htags, HTTP_HEADER_CONTENT_TYPE)) {
-		const buffer *content_type = stat_cache_content_type_get(sce, r);
-		if (content_type && !buffer_is_blank(content_type)) {
-			http_header_response_set(r, HTTP_HEADER_CONTENT_TYPE,
-			                         CONST_STR_LEN("Content-Type"),
-			                         BUF_PTR_LEN(content_type));
-		} else {
-			/* we are setting application/octet-stream, but also announce that
-			 * this header field might change in the seconds few requests
-			 *
-			 * This should fix the aggressive caching of FF and the script download
-			 * seen by the first installations
-			 */
-			http_header_response_set(r, HTTP_HEADER_CONTENT_TYPE,
-			                         CONST_STR_LEN("Content-Type"),
-			                         CONST_STR_LEN("application/octet-stream"));
-
-			allow_caching = 0;
-		}
+	static const buffer octet_stream =
+	  { CONST_STR_LEN("application/octet-stream")+1, 0 };
+	const buffer *content_type = NULL;
+	if (__builtin_expect( (!light_btst(r->resp_htags, HTTP_HEADER_CONTENT_TYPE)), 1)) {
+		content_type = stat_cache_content_type_get(sce, r);
+		if (__builtin_expect( (!content_type), 0)
+		    || __builtin_expect( (buffer_is_blank(content_type)), 0))
+			content_type = &octet_stream;
+		http_header_response_set(r, HTTP_HEADER_CONTENT_TYPE,
+		                         CONST_STR_LEN("Content-Type"),
+		                         BUF_PTR_LEN(content_type));
 	}
 
-	if (allow_caching) {
-		if (!light_btst(r->resp_htags, HTTP_HEADER_ETAG)
+	/* avoid sending caching headers if implicit "application/octet-stream"
+	 * This should workaround aggressive caching by FF and script download
+	 * seen by the first installations (e.g. if lighttpd is misconfigured)*/
+	int allow_caching = (content_type != &octet_stream)
+	                 && (0 == r->http_status || 200 == r->http_status);
+	if (__builtin_expect( (allow_caching), 1)) {
+		if (__builtin_expect( (!light_btst(r->resp_htags, HTTP_HEADER_ETAG)), 1)
 		    && 0 != r->conf.etag_flags) {
 			const buffer *etag =
 			  stat_cache_etag_get(sce, r->conf.etag_flags);
 			if (etag && !buffer_is_blank(etag)) {
-				buffer_copy_buffer(&r->physical.etag, etag);
 				http_header_response_set(r, HTTP_HEADER_ETAG,
 				                         CONST_STR_LEN("ETag"),
-				                         BUF_PTR_LEN(&r->physical.etag));
+				                         BUF_PTR_LEN(etag));
 			}
 		}
 
-		/* prepare header */
-		const buffer *mtime;
-		mtime = http_header_response_get(r, HTTP_HEADER_LAST_MODIFIED,
-		                                 CONST_STR_LEN("Last-Modified"));
-		if (NULL == mtime) {
-			mtime = http_response_set_last_modified(r, sce->st.st_mtime);
-		}
+		const buffer * const lmod =
+		  __builtin_expect( (!light_btst(r->resp_htags, HTTP_HEADER_LAST_MODIFIED)), 1)
+		  ? http_response_set_last_modified(r, sce->st.st_mtime)
+		  : NULL;
 
-		if (HANDLER_FINISHED == http_response_handle_cachable(r, mtime, sce->st.st_mtime)) {
+		if (http_response_maybe_cachable(r)
+		    && HANDLER_FINISHED == http_response_handle_cachable(r, lmod, sce->st.st_mtime))
 			return;
-		}
 	}
 
 	/* if we are still here, prepare body */
