@@ -1601,8 +1601,8 @@ static int mod_ssi_handle_request(request_st * const r, handler_ctx * const p) {
 		if (TIME64_CAST(st.st_mtime) < include_file_last_mtime)
 			st.st_mtime = include_file_last_mtime;
 
-		http_etag_create(&r->physical.etag, &st, r->conf.etag_flags);
-		http_header_response_set(r, HTTP_HEADER_ETAG, CONST_STR_LEN("ETag"), BUF_PTR_LEN(&r->physical.etag));
+		http_etag_create(r->tmp_buf, &st, r->conf.etag_flags);
+		http_header_response_set(r, HTTP_HEADER_ETAG, CONST_STR_LEN("ETag"), BUF_PTR_LEN(r->tmp_buf));
 
 		const buffer * const mtime = http_response_set_last_modified(r, st.st_mtime);
 		if (HANDLER_FINISHED == http_response_handle_cachable(r, mtime, st.st_mtime)) {
