@@ -321,7 +321,7 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 	srv_socket->sidx = sidx;
 	srv_socket->is_ssl = s->ssl_enabled;
 	srv_socket->srv = srv;
-	srv_socket->srv_token = buffer_init_buffer(host_token);
+	buffer_copy_buffer((srv_socket->srv_token = buffer_init()), host_token);
 	srv_socket->srv_token_colon =
 	  network_srv_token_colon(srv_socket->srv_token);
 
@@ -442,19 +442,21 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 	}
 
 	if (s->ssl_enabled) {
+	}
 #ifdef TCP_DEFER_ACCEPT
-	} else if (s->defer_accept) {
+	else if (s->defer_accept) {
 		int v = s->defer_accept;
 		if (-1 == setsockopt(srv_socket->fd, IPPROTO_TCP, TCP_DEFER_ACCEPT, &v, sizeof(v))) {
 			log_perror(srv->errh, __FILE__, __LINE__, "can't set TCP_DEFER_ACCEPT");
 		}
+	}
 #endif
 #if defined(__FreeBSD__) || defined(__NetBSD__) \
  || defined(__OpenBSD__) || defined(__DragonFly__)
-	} else if (s->bsd_accept_filter
+#ifdef SO_ACCEPTFILTER
+	else if (s->bsd_accept_filter
 		   && (buffer_is_equal_string(s->bsd_accept_filter, CONST_STR_LEN("httpready"))
 			|| buffer_is_equal_string(s->bsd_accept_filter, CONST_STR_LEN("dataready")))) {
-#ifdef SO_ACCEPTFILTER
 		/* FreeBSD accf_http filter */
 		struct accept_filter_arg afa;
 		memset(&afa, 0, sizeof(afa));
@@ -465,9 +467,9 @@ static int network_server_init(server *srv, network_socket_config *s, buffer *ho
 				  "can't set accept-filter '%s'", s->bsd_accept_filter->ptr);
 			}
 		}
-#endif
-#endif
 	}
+#endif
+#endif
 
 	return 0;
 }
@@ -579,7 +581,13 @@ static int network_socket_activation_from_env(server *srv, network_socket_config
     char *listen_fds = getenv("LISTEN_FDS");
     pid_t lpid = listen_pid ? (pid_t)strtoul(listen_pid,NULL,10) : 0;
     int nfds = listen_fds ? atoi(listen_fds) : 0;
-    int rc = (lpid == getpid() && nfds > 0 && nfds < 5000)
+    int rc = (nfds > 0 && nfds < 5000
+              && (lpid == getpid()
+                 #ifndef _WIN32
+                  || (0 == strncmp(listen_pid, "parent:", 7)
+                      && getppid() == (pid_t)strtoul(listen_pid+7,NULL,10))
+                 #endif
+                 ))
       ? network_socket_activation_nfds(srv, s, nfds)
       : 0;
     unsetenv("LISTEN_PID");
@@ -756,8 +764,9 @@ int network_init(server *srv, int stdin_fd) {
                 force_assert(NULL != srv_socket);
                 memcpy(srv_socket, srv->srv_sockets_inherited.ptr[i],
                        sizeof(server_socket));
-                srv_socket->srv_token =
-                  buffer_init_buffer(srv_socket->srv_token);
+                const buffer * const srv_token = srv_socket->srv_token;
+                buffer_copy_buffer((srv_socket->srv_token = buffer_init()),
+                                   srv_token);
                 srv_socket->srv_token_colon =
                   network_srv_token_colon(srv_socket->srv_token);
                 network_srv_sockets_append(srv, srv_socket);
