@@ -65,6 +65,12 @@ static const buffer default_server_tag = { CONST_STR_LEN(PACKAGE_DESC)+1, 0 };
 #ifdef HAVE_SYS_PRCTL_H
 # include <sys/prctl.h>
 #endif
+#ifdef HAVE_SYS_PROCCTL_H
+# include <sys/procctl.h>
+#endif
+#ifdef HAVE_PRIV_H
+# include <priv.h>
+#endif
 
 #ifdef HAVE_MALLOC_H
 #ifndef LIGHTTPD_STATIC
@@ -247,18 +253,21 @@ server_monotonic_secs (void)
 }
 
 static unix_time64_t
-server_epoch_secs (server * const srv)
+server_epoch_secs (server * const srv, unix_time64_t mono_ts_delta)
 {
     const unix_time64_t cur_ts = log_epoch_secs;
     const unix_time64_t new_ts = TIME64_CAST(time(NULL));
+    const unix_time64_t new_ts_adj = new_ts - mono_ts_delta;
     /* attempt to detect large clock jump */
-    if (new_ts < cur_ts || new_ts - cur_ts > 300) { /*(5 mins)*/
+    if (new_ts_adj < cur_ts || new_ts_adj - cur_ts > 300) { /*(5 mins)*/
         log_error(srv->errh, __FILE__, __LINE__,
           "warning: clock jumped %lld secs",
-          (long long)((int64_t)new_ts - (int64_t)cur_ts));
+          (long long)((int64_t)new_ts_adj - (int64_t)cur_ts));
         int delta =                             /*(30 mins default)*/
           config_feature_int(srv, "server.clock-jump-restart", 1800);
-        if (delta && (new_ts > cur_ts ? new_ts-cur_ts : cur_ts-new_ts) > delta){
+        if (delta && (new_ts_adj > cur_ts
+                      ? new_ts_adj-cur_ts
+                      : cur_ts-new_ts_adj) > delta) {
             log_error(srv->errh, __FILE__, __LINE__,
               "attempting graceful restart in < ~5 seconds, else hard restart");
             srv->graceful_expire_ts = log_monotonic_secs + 5;
@@ -627,7 +636,7 @@ static void show_features (void) {
 #else
       "\t- brotli support\n"
 #endif
-#if defined(HAVE_CRYPT) || defined(HAVE_CRYPT_R) || defined(HAVE_LIBCRYPT)
+#if defined(HAVE_CRYPT) || defined(HAVE_CRYPT_R)
       "\t+ crypt support\n"
 #else
       "\t- crypt support\n"
@@ -697,11 +706,6 @@ static void show_features (void) {
 #else
       "\t- PAM support\n"
 #endif
-#ifdef USE_MEMCACHED
-      "\t+ memcached support\n"
-#else
-      "\t- memcached support\n"
-#endif
 #ifdef HAVE_FAM_H
       "\t+ FAM support\n"
 #else
@@ -721,11 +725,6 @@ static void show_features (void) {
       "\t+ SQLite support\n"
 #else
       "\t- SQLite support\n"
-#endif
-#ifdef HAVE_GDBM_H
-      "\t+ GDBM support\n"
-#else
-      "\t- GDBM support\n"
 #endif
       ;
   show_version();
@@ -997,7 +996,7 @@ static void server_graceful_state (server *srv) {
     if (!srv_shutdown) {
         if (0 == srv->graceful_expire_ts) {
             srv->graceful_expire_ts =
-              config_feature_int(srv, "server.graceful-shutdown-timeout", 0);
+              config_feature_int(srv, "server.graceful-shutdown-timeout", 8);
             if (srv->graceful_expire_ts)
                 srv->graceful_expire_ts += log_monotonic_secs;
         }
@@ -1111,7 +1110,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 			}
 			break;
 		case 'm':
-			buffer_copy_string(srv->srvconf.modules_dir, optarg);
+			srv->srvconf.modules_dir = optarg;
 			break;
 		case 'i': {
 			char *endptr;
@@ -1146,7 +1145,7 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 			return -1;
 		}
 		srv->srvconf.dont_daemonize = 1;
-		buffer_copy_string_len(srv->srvconf.modules_dir, CONST_STR_LEN("modules"));
+		srv->srvconf.modules_dir = "modules";
 		if (config_read(srv, "conf/lighttpd.conf")) return -1;
 	}
       #endif
@@ -1349,12 +1348,16 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		    && (i_am_root || srv->srvconf.max_fds <= rlim.rlim_max)) {
 			/* set rlimits */
 
+			rlim_t rlim_cur = rlim.rlim_cur;
 			rlim.rlim_cur = srv->srvconf.max_fds;
 			if (i_am_root) rlim.rlim_max = srv->srvconf.max_fds;
 
 			if (0 != setrlimit(RLIMIT_NOFILE, &rlim)) {
 				log_perror(srv->errh, __FILE__, __LINE__, "setrlimit()");
-				return -1;
+				log_error(srv->errh, __FILE__, __LINE__, "setrlimit() may need root to run once: setsebool -P httpd_setrlimit on");
+				use_rlimit = 0;
+				if (srv->srvconf.max_fds > rlim_cur)
+					srv->srvconf.max_fds = rlim_cur;
 			}
 		}
 
@@ -1468,6 +1471,20 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 		if (srv->srvconf.enable_cores) {
 			prctl(PR_SET_DUMPABLE, 1, 0, 0, 0);
 		}
+#elif defined(HAVE_SYS_PROCCTL_H) && defined(PROC_TRACE_CTL_ENABLE)
+		/* (DragonFlyBSD has procctl(), but not PROC_TRACE_CTL_ENABLE) */
+		if (srv->srvconf.enable_cores) {
+			int dumpable = PROC_TRACE_CTL_ENABLE;
+			procctl(P_PID, 0, PROC_TRACE_CTL, &dumpable);
+		}
+#elif defined(HAVE_SETPFLAGS) && defined(__PROC_PROTECT)
+		/**
+		 * setpflags seems uniquely a solaris/illumos feature
+		 * but just taking extra precautions clearing __PROC_PROTECT option
+		 */
+		if (srv->srvconf.enable_cores) {
+			setpflags(__PROC_PROTECT, 0);
+		}
 #endif
 	}
 
@@ -1613,10 +1630,12 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 				}
 			} else {
 				int status;
+				unix_time64_t mono_ts;
 
 				if (-1 != (pid = fdevent_waitpid_intr(-1, &status))) {
+					mono_ts = log_monotonic_secs;
 					log_monotonic_secs = server_monotonic_secs();
-					log_epoch_secs = server_epoch_secs(srv);
+					log_epoch_secs = server_epoch_secs(srv, log_monotonic_secs - mono_ts);
 					if (plugins_call_handle_waitpid(srv, pid, status) != HANDLER_GO_ON) {
 						if (!timer) alarm((timer = 5));
 						continue;
@@ -1640,8 +1659,9 @@ static int server_main_setup (server * const srv, int argc, char **argv) {
 				} else {
 					switch (errno) {
 					case EINTR:
+						mono_ts = log_monotonic_secs;
 						log_monotonic_secs = server_monotonic_secs();
-						log_epoch_secs = server_epoch_secs(srv);
+						log_epoch_secs = server_epoch_secs(srv, log_monotonic_secs - mono_ts);
 						/**
 						 * if we receive a SIGHUP we have to close our logs ourself as we don't 
 						 * have the mainloop who can help us here
@@ -1840,7 +1860,7 @@ static void server_handle_sigalrm (server * const srv, unix_time64_t mono_ts, un
 				plugins_call_handle_trigger(srv);
 
 				log_monotonic_secs = mono_ts;
-				log_epoch_secs = server_epoch_secs(srv);
+				log_epoch_secs = server_epoch_secs(srv, 0);
 
 				/* check idle time limit, if enabled */
 				if (idle_limit && idle_limit < mono_ts - last_active_ts && !graceful_shutdown) {
@@ -1923,7 +1943,7 @@ __attribute_hot__
 __attribute_noinline__
 static void server_main_loop (server * const srv) {
 	unix_time64_t last_active_ts = server_monotonic_secs();
-	log_epoch_secs = server_epoch_secs(srv);
+	log_epoch_secs = server_epoch_secs(srv, 0);
 
 	while (!srv_shutdown) {
 
